@@ -1,12 +1,14 @@
 # coding:utf-8
 
+from cStringIO import StringIO
 import zlib
 
-from nose.tools import *
 from mock import *
-from flanker.mime.message.headers import MimeHeaders
+from nose.tools import *
+
 from flanker.mime.message.errors import DecodingError
-from cStringIO import StringIO
+from flanker.mime.message.headers import encoding
+from flanker.mime.message.headers import MimeHeaders
 
 from .... import *
 
@@ -119,6 +121,14 @@ def headers_transform_test():
 
     eq_([('X-Mime-Version', 't(1)'), ('X-Received', 't(2)'), ('X-Mime-Version', 't(3)'), ('X-Received', 't(4)')], h.items())
 
+def headers_transform_encodedword_test():
+    # Create a header with non-ascii characters that will be stored in encoded-word format.
+    headers = [('Subject', encoding.to_mime('Subject', u'Hello ✓'))]
+    h = MimeHeaders(headers)
+
+    # transform should decode it for us when we pass decode=True
+    h.transform(lambda key,val: (key, val.replace(u'✓', u'☃')), decode=True)
+    eq_(u'Hello ☃', h.get('Subject'))
 
 def headers_parsing_empty_test():
     h = MimeHeaders.from_stream(StringIO(""))
@@ -134,8 +144,7 @@ def headers_parsing_ridiculously_long_line_test():
 def headers_parsing_binary_stuff_survives_test():
     value = zlib.compress("abcdefg")
     header = "Hello: {0}\r\n".format(value)
-    assert_raises(
-        DecodingError, MimeHeaders.from_stream, StringIO(header))
+    ok_(MimeHeaders.from_stream(StringIO(header)))
 
 
 def broken_sequences_test():
@@ -171,7 +180,6 @@ def headers_roundtrip_test():
         headers2['DKIM-Signature'])
 
 
-
 def test_folding_combinations():
     message = """From mrc@example.com Mon Feb  8 02:53:47 PST 1993\nTo: sasha\r\n  continued\n      line\nFrom: single line  \r\nSubject: hello, how are you\r\n today?"""
     headers = MimeHeaders.from_stream(StringIO(message))
